@@ -19,23 +19,25 @@
 #include "usb.h"
 #include "loader.h"
 
-extern KSPIN_LOCK ntoskernel_lock;
-extern KSPIN_LOCK loader_lock;
-extern KSPIN_LOCK irp_cancel_lock;
+extern NT_SPIN_LOCK ntoskernel_lock;
+extern NT_SPIN_LOCK loader_lock;
+extern NT_SPIN_LOCK irp_cancel_lock;
 extern struct nt_list object_list;
 
-extern KSPIN_LOCK irp_cancel_lock;
+extern NT_SPIN_LOCK irp_cancel_lock;
 
 STDCALL void WRAP_EXPORT(IoAcquireCancelSpinLock)
 	(KIRQL *irql)
 {
-	*irql = kspin_lock_irql(&irp_cancel_lock, DISPATCH_LEVEL);
+	*irql = raise_irql(DISPATCH_LEVEL);
+	nt_spin_lock(&irp_cancel_lock);
 }
 
 STDCALL void WRAP_EXPORT(IoReleaseCancelSpinLock)
 	(KIRQL irql)
 {
-	kspin_unlock_irql(&irp_cancel_lock, irql);
+	lower_irql(irql);
+	nt_spin_unlock(&irp_cancel_lock);
 }
 
 STDCALL NTSTATUS WRAP_EXPORT(IoGetDeviceProperty)
@@ -144,7 +146,7 @@ STDCALL BOOLEAN WRAP_EXPORT(IoIs32bitProcess)
 STDCALL void WRAP_EXPORT(IoInitializeIrp)
 	(struct irp *irp, USHORT size, CCHAR stack_size)
 {
-	IOENTER("irp = %p, stack_size = %d", irp, stack_size);
+	IOENTER("irp: %p, stack_size: %d", irp, stack_size);
 
 	memset(irp, 0, size);
 	irp->size = size;
@@ -157,7 +159,7 @@ STDCALL void WRAP_EXPORT(IoInitializeIrp)
 STDCALL void WRAP_EXPORT(IoReuseIrp)
 	(struct irp *irp, NTSTATUS status)
 {
-	IOENTER("irp = %p, status = %d", irp, status);
+	IOENTER("irp: %p, status: %d", irp, status);
 	if (irp) {
 		UCHAR alloc_flags;
 
@@ -175,8 +177,7 @@ STDCALL struct irp *WRAP_EXPORT(IoAllocateIrp)
 	struct irp *irp;
 	int irp_size;
 
-	IOENTER("stack_size = %d, charge_quota = %d",
-		stack_size, charge_quota);
+	IOENTER("stack_size: %d, charge_quota: %d", stack_size, charge_quota);
 
 	irp_size = IoSizeOfIrp(stack_size + 1);
 	irp = kmalloc(irp_size, GFP_ATOMIC);
@@ -184,14 +185,6 @@ STDCALL struct irp *WRAP_EXPORT(IoAllocateIrp)
 		IOTRACE("allocated irp %p", irp);
 		IoInitializeIrp(irp, irp_size, stack_size);
 	}
-#if 0
-	DBG_BLOCK() {
-		int i;
-		for (i = 0; i < stack_size; i++)
-			INFO("stack %d at %p", i, IRP_SL(irp, i));
-	}
-#endif
-
 	IOEXIT(return irp);
 }
 
@@ -231,16 +224,16 @@ STDCALL void IoQueueThreadIrp(struct irp *irp)
 	thread = KeGetCurrentThread();
 	if (!thread) {
 		WARNING("couldn't find thread for irp: %p, task: %p, pid: %d",
-			irp, get_current(), get_current()->pid);
+			irp, current, current->pid);
 		IoIrpThread(irp) = NULL;
 		return;
 	}
 	IOTRACE("thread: %p, task: %p", thread, thread->task);
-	irql = kspin_lock_irql(&thread->lock, DISPATCH_LEVEL);
+	irql = nt_spin_lock_irql(&thread->lock, DISPATCH_LEVEL);
 	irp->flags |= IRP_SYNCHRONOUS_API;
 	InsertTailList(&thread->irps, &irp->threads);
 	IoIrpThread(irp) = thread;
-	kspin_unlock_irql(&thread->lock, irql);
+	nt_spin_unlock_irql(&thread->lock, irql);
 }
 
 STDCALL void IoDequeueThreadIrp(struct irp *irp)
@@ -250,9 +243,9 @@ STDCALL void IoDequeueThreadIrp(struct irp *irp)
 
 	thread = IoIrpThread(irp);
 	if (thread) {
-		irql = kspin_lock_irql(&thread->lock, DISPATCH_LEVEL);
+		irql = nt_spin_lock_irql(&thread->lock, DISPATCH_LEVEL);
 		RemoveEntryList(&irp->threads);
-		kspin_unlock_irql(&thread->lock, irql);
+		nt_spin_unlock_irql(&thread->lock, irql);
 	}
 }
 
@@ -384,15 +377,12 @@ _FASTCALL NTSTATUS WRAP_EXPORT(IofCallDriver)
 	driver_dispatch_t *major_func;
 	struct driver_object *drv_obj;
 
-	DUMP_IRP(irp);
-	if (!dev_obj)
-		IOEXIT(return STATUS_SUCCESS);
-	drv_obj = dev_obj->drv_obj;
-	IOTRACE("drv_obj: %p", drv_obj);
 	IoSetNextIrpStackLocation(irp);
+	DUMP_IRP(irp);
 	irp_sl = IoGetCurrentIrpStackLocation(irp);
-
+	drv_obj = dev_obj->drv_obj;
 	irp_sl->dev_obj = dev_obj;
+	IOTRACE("drv_obj: %p", drv_obj);
 	major_func = drv_obj->major_func[irp_sl->major_fn];
 	IOTRACE("major_func: %p, dev_obj: %p", major_func, dev_obj);
 	if (major_func)
@@ -412,6 +402,7 @@ _FASTCALL void WRAP_EXPORT(IofCompleteRequest)
 	struct io_stack_location *irp_sl;
 	struct mdl *mdl;
 
+#ifdef IO_DEBUG
 	DUMP_IRP(irp);
 	if (irp->io_status.status == STATUS_PENDING) {
 		ERROR("invalid irp: %p, STATUS_PENDING", irp);
@@ -421,8 +412,12 @@ _FASTCALL void WRAP_EXPORT(IofCompleteRequest)
 		ERROR("invalid irp: %p, %d", irp, irp->current_location);
 		return;
 	}
-
+#endif
 	irp_sl = IoGetCurrentIrpStackLocation(irp);
+	/* completion routines expect current stack location to match
+	 * with what was set through IoSetCompletionRoutine, which
+	 * sets completion routine at next (higher) location, so we
+	 * skip current location and continue past stack size */
 	IoSkipCurrentIrpStackLocation(irp);
 
 	while (irp->current_location <= (irp->stack_count + 1)) {
@@ -482,7 +477,7 @@ _FASTCALL void WRAP_EXPORT(IofCompleteRequest)
 static irqreturn_t io_irq_th(int irq, void *data, struct pt_regs *pt_regs)
 {
 	struct kinterrupt *interrupt = (struct kinterrupt *)data;
-	KSPIN_LOCK *spinlock;
+	NT_SPIN_LOCK *spinlock;
 	BOOLEAN ret;
 	KIRQL irql = PASSIVE_LEVEL;
 
@@ -491,15 +486,15 @@ static irqreturn_t io_irq_th(int irq, void *data, struct pt_regs *pt_regs)
 	else
 		spinlock = &interrupt->lock;
 	if (interrupt->synch_irql >= DISPATCH_LEVEL)
-		irql = kspin_lock_irql(spinlock, DISPATCH_LEVEL);
+		irql = nt_spin_lock_irql(spinlock, DISPATCH_LEVEL);
 	else
-		kspin_lock(spinlock);
+		nt_spin_lock(spinlock);
 	ret = interrupt->service_routine(interrupt,
 					 interrupt->service_context);
 	if (interrupt->synch_irql >= DISPATCH_LEVEL)
-		kspin_unlock_irql(spinlock, irql);
+		nt_spin_unlock_irql(spinlock, irql);
 	else
-		kspin_unlock(spinlock);
+		nt_spin_unlock(spinlock);
 
 	if (ret == TRUE)
 		return IRQ_HANDLED;
@@ -509,7 +504,7 @@ static irqreturn_t io_irq_th(int irq, void *data, struct pt_regs *pt_regs)
 
 STDCALL NTSTATUS WRAP_EXPORT(IoConnectInterrupt)
 	(struct kinterrupt *interrupt, PKSERVICE_ROUTINE service_routine,
-	 void *service_context, KSPIN_LOCK *lock, ULONG vector,
+	 void *service_context, NT_SPIN_LOCK *lock, ULONG vector,
 	 KIRQL irql, KIRQL synch_irql, enum kinterrupt_mode interrupt_mode,
 	 BOOLEAN shareable, KAFFINITY processor_enable_mask,
 	 BOOLEAN floating_save)
@@ -518,7 +513,7 @@ STDCALL NTSTATUS WRAP_EXPORT(IoConnectInterrupt)
 
 	interrupt->vector = vector;
 	interrupt->processor_enable_mask = processor_enable_mask;
-	kspin_lock_init(&interrupt->lock);
+	nt_spin_lock_init(&interrupt->lock);
 	interrupt->actual_lock = lock;
 	interrupt->shareable = shareable;
 	interrupt->floating_save = floating_save;
@@ -628,9 +623,9 @@ STDCALL NTSTATUS WRAP_EXPORT(IoAllocateDriverObjectExtension)
 
 	IOTRACE("custom_ext: %p", ce);
 	ce->client_id = client_id;
-	irql = kspin_lock_irql(&ntoskernel_lock, DISPATCH_LEVEL);
+	irql = nt_spin_lock_irql(&ntoskernel_lock, DISPATCH_LEVEL);
 	InsertTailList(&drv_obj->drv_ext->custom_ext, &ce->list);
-	kspin_unlock_irql(&ntoskernel_lock, irql);
+	nt_spin_unlock_irql(&ntoskernel_lock, irql);
 
 	*ext = (void *)ce + sizeof(*ce);
 	IOTRACE("ext: %p", *ext);
@@ -646,14 +641,14 @@ STDCALL void *WRAP_EXPORT(IoGetDriverObjectExtension)
 
 	IOENTER("drv_obj: %p, client_id: %p", drv_obj, client_id);
 	ret = NULL;
-	irql = kspin_lock_irql(&ntoskernel_lock, DISPATCH_LEVEL);
+	irql = nt_spin_lock_irql(&ntoskernel_lock, DISPATCH_LEVEL);
 	nt_list_for_each_entry(ce, &drv_obj->drv_ext->custom_ext, list) {
 		if (ce->client_id == client_id) {
 			ret = (void *)ce + sizeof(*ce);
 			break;
 		}
 	}
-	kspin_unlock_irql(&ntoskernel_lock, irql);
+	nt_spin_unlock_irql(&ntoskernel_lock, irql);
 	IOENTER("ret: %p", ret);
 	IOEXIT(return ret);
 }
@@ -664,10 +659,10 @@ void free_custom_extensions(struct driver_extension *drv_ext)
 	KIRQL irql;
 
 	IOENTER("%p", drv_ext);
-	irql = kspin_lock_irql(&ntoskernel_lock, DISPATCH_LEVEL);
+	irql = nt_spin_lock_irql(&ntoskernel_lock, DISPATCH_LEVEL);
 	while ((ent = RemoveHeadList(&drv_ext->custom_ext)))
 		kfree(ent);
-	kspin_unlock_irql(&ntoskernel_lock, irql);
+	nt_spin_unlock_irql(&ntoskernel_lock, irql);
 	IOEXIT(return);
 }
 
@@ -798,10 +793,10 @@ STDCALL struct device_object *WRAP_EXPORT(IoGetAttachedDevice)
 	IOENTER("%p", dev);
 	if (!dev)
 		IOEXIT(return NULL);
-	irql = kspin_lock_irql(&ntoskernel_lock, DISPATCH_LEVEL);
+	irql = nt_spin_lock_irql(&ntoskernel_lock, DISPATCH_LEVEL);
 	for (d = dev; d->attached; d = d->attached)
 		;
-	kspin_unlock_irql(&ntoskernel_lock, irql);
+	nt_spin_unlock_irql(&ntoskernel_lock, irql);
 	IOEXIT(return d);
 }
 
@@ -828,13 +823,13 @@ STDCALL struct device_object *WRAP_EXPORT(IoAttachDeviceToDeviceStack)
 	dst = IoGetAttachedDevice(tgt);
 	IOTRACE("stack_size: %d -> %d", dst->stack_size, src->stack_size);
 	IOTRACE("%p", dst);
-	irql = kspin_lock_irql(&ntoskernel_lock, DISPATCH_LEVEL);
+	irql = nt_spin_lock_irql(&ntoskernel_lock, DISPATCH_LEVEL);
 	if (dst)
 		dst->attached = src;
 	src->attached = NULL;
 	src->stack_size = dst->stack_size + 1;
 //	src->ref_count++;
-	kspin_unlock_irql(&ntoskernel_lock, irql);
+	nt_spin_unlock_irql(&ntoskernel_lock, irql);
 	IOTRACE("stack_size: %d -> %d", dst->stack_size, src->stack_size);
 	IOEXIT(return dst);
 }
@@ -853,14 +848,14 @@ STDCALL void WRAP_EXPORT(IoDetachDevice)
 		IOEXIT(return);
 	IOTRACE("tail: %p", tail);
 
-	irql = kspin_lock_irql(&ntoskernel_lock, DISPATCH_LEVEL);
+	irql = nt_spin_lock_irql(&ntoskernel_lock, DISPATCH_LEVEL);
 	topdev->attached = tail->attached;
 	IOTRACE("attached:%p", topdev->attached);
 	for ( ; tail; tail = tail->attached) {
 		IOTRACE("tail:%p", tail);
 		tail->stack_size--;
 	}
-	kspin_unlock_irql(&ntoskernel_lock, irql);
+	nt_spin_unlock_irql(&ntoskernel_lock, irql);
 	IOEXIT(return);
 }
 
