@@ -158,17 +158,19 @@ static int check_nt_hdr(IMAGE_NT_HEADERS *nt_hdr)
 #endif
 
 	/* Validate the image for the current architecture. */
-	if (nt_hdr->FileHeader.Machine !=
 #ifdef CONFIG_X86_64
-	    IMAGE_FILE_MACHINE_AMD64
-#else
-	    IMAGE_FILE_MACHINE_I386
-#endif
-		) {
-		ERROR("driver is not for current architecture "
+	if (nt_hdr->FileHeader.Machine != IMAGE_FILE_MACHINE_AMD64) {
+		ERROR("Windows driver is not 64-bit;"
 		      " (PE signature is %04X)", nt_hdr->FileHeader.Machine);
 		return -EINVAL;
 	}
+#else
+	if (nt_hdr->FileHeader.Machine != IMAGE_FILE_MACHINE_I386) {
+		ERROR("Windows driver is not 32-bit;"
+		      " (PE signature is %04X)", nt_hdr->FileHeader.Machine);
+		return -EINVAL;
+	}
+#endif
 
 	/* Must have attributes */
 #ifdef CONFIG_X86_64
@@ -323,7 +325,7 @@ static int fixup_imports(void *image, IMAGE_NT_HEADERS *nt_hdr)
 
 static int fixup_reloc(void *image, IMAGE_NT_HEADERS *nt_hdr)
 {
-        ULONG_PTR base;
+	ULONG_PTR base;
 	ULONG_PTR size;
 	IMAGE_BASE_RELOCATION *fixup_block;
 	IMAGE_DATA_DIRECTORY *base_reloc_data_dir;
@@ -331,7 +333,7 @@ static int fixup_reloc(void *image, IMAGE_NT_HEADERS *nt_hdr)
 
 	opt_hdr = &nt_hdr->OptionalHeader;
 	base = opt_hdr->ImageBase;
-	base_reloc_data_dir = 
+	base_reloc_data_dir =
 		&opt_hdr->DataDirectory[IMAGE_DIRECTORY_ENTRY_BASERELOC];
 	if (base_reloc_data_dir->Size == 0)
 		return 0;
@@ -339,7 +341,7 @@ static int fixup_reloc(void *image, IMAGE_NT_HEADERS *nt_hdr)
 	fixup_block = RVA2VA(image, base_reloc_data_dir->VirtualAddress,
 			     IMAGE_BASE_RELOCATION *);
 	DBGLINKER("fixup_block=%p, image=%p", fixup_block, image);
-	DBGLINKER("fixup_block info: %x %d", 
+	DBGLINKER("fixup_block info: %x %d",
 		  fixup_block->VirtualAddress, fixup_block->SizeOfBlock);
 
 	while (fixup_block->SizeOfBlock) {
@@ -532,7 +534,7 @@ int load_pe_images(struct pe_image *pe_image, int n)
 
 		if (pe->size < sizeof(IMAGE_DOS_HEADER)) {
 			DBGTRACE1("image too small: %d", pe->size);
- 			return -EINVAL;
+			return -EINVAL;
 		}
 
 		pe->nt_hdr =
@@ -576,7 +578,7 @@ int load_pe_images(struct pe_image *pe_image, int n)
 		pe->entry =
 			RVA2VA(pe->image,
 			       pe->opt_hdr->AddressOfEntryPoint, void *);
-		DBGTRACE1("entry is at %p, rva at %08X", pe->entry, 
+		DBGTRACE1("entry is at %p, rva at %08X", pe->entry,
 			  pe->opt_hdr->AddressOfEntryPoint);
 	}
 
