@@ -108,7 +108,7 @@ static void usb_init_urb(struct urb *urb)
 
 #endif
 
-extern KSPIN_LOCK irp_cancel_lock;
+extern NT_SPIN_LOCK irp_cancel_lock;
 
 /* TODO: using a worker instead of tasklet is probably preferable, but
  * with ZyDas driver the worker never gets executed, even though it
@@ -149,10 +149,9 @@ void usb_exit_device(struct wrap_device *wd)
 	struct wrap_urb *wrap_urb;
 	KIRQL irql;
 
+	IoAcquireCancelSpinLock(&irql);
 	while (1) {
-		IoAcquireCancelSpinLock(&irql);
 		ent = RemoveHeadList(&wd->usb.wrap_urb_list);
-		IoReleaseCancelSpinLock(irql);
 		if (!ent)
 			break;
 		wrap_urb = container_of(ent, struct wrap_urb, list);
@@ -172,6 +171,7 @@ void usb_exit_device(struct wrap_device *wd)
 	wd->usb.num_alloc_urbs = 0;
 	wd->usb.udev = NULL;
 	wd->usb.intf = NULL;
+	IoReleaseCancelSpinLock(irql);
 	return;
 }
 
@@ -244,7 +244,7 @@ static void wrap_free_urb(struct urb *urb)
 		USBTRACE("freeing DMA buffer for URB: %p %p",
 			 urb, urb->transfer_buffer);
 		usb_buffer_free(irp->wd->usb.udev,
-				urb->transfer_buffer_length, 
+				urb->transfer_buffer_length,
 				urb->transfer_buffer, urb->transfer_dma);
 	}
 	if (urb->setup_packet)
@@ -484,10 +484,9 @@ static void irp_complete_worker(unsigned long data)
 		switch (URB_STATUS(wrap_urb)) {
 		case 0:
 			/* succesfully transferred */
-			NT_URB_STATUS(nt_urb) =
-				wrap_urb_status(URB_STATUS(wrap_urb));
 			irp->io_status.status = STATUS_SUCCESS;
 			irp->io_status.status_info = urb->actual_length;
+			NT_URB_STATUS(nt_urb) = USBD_STATUS_SUCCESS;
 
 			/* from WDM examples, it seems we don't need
 			 * to update MDL's byte count if MDL is used */
@@ -650,14 +649,13 @@ static USBD_STATUS wrap_bulk_or_intr_trans(struct irp *irp)
 
 static USBD_STATUS wrap_vendor_or_class_req(struct irp *irp)
 {
-	struct urb *urb;
-	struct usb_ctrlrequest *dr;
 	char req_type;
 	unsigned int pipe;
 	struct usbd_vendor_or_class_request *vc_req;
 	struct usb_device *udev;
 	union nt_urb *nt_urb;
 	USBD_STATUS status;
+	int ret;
 
 	nt_urb = URB_FROM_IRP(irp);
 	udev = irp->wd->usb.udev;
@@ -721,39 +719,56 @@ static USBD_STATUS wrap_vendor_or_class_req(struct irp *irp)
 		req_type |= USB_DIR_OUT;
 		USBTRACE("pipe: %u, dir out", pipe);
 	}
+	if (current_irql() < DISPATCH_LEVEL) {
+		/* synchronous submission */
+		ret = usb_control_msg(udev, pipe, vc_req->request, req_type,
+				      vc_req->value, vc_req->index,
+				      vc_req->transfer_buffer,
+				      vc_req->transfer_buffer_length, 0);
+		USBTRACE("ret: %d", ret);
+		if (ret < 0)
+			status = wrap_urb_status(ret);
+		else {
+			vc_req->transfer_buffer_length = ret;
+			status = USBD_STATUS_SUCCESS;
+		}
+	} else {
+		/* asynchronous submission */
+		struct urb *urb;
+		struct usb_ctrlrequest *dr;
+		urb = wrap_alloc_urb(irp, pipe, vc_req->transfer_buffer,
+				     vc_req->transfer_buffer_length);
+		if (!urb) {
+			ERROR("couldn't allocate urb");
+			return USBD_STATUS_NO_MEMORY;
+		}
 
-	urb = wrap_alloc_urb(irp, pipe, vc_req->transfer_buffer,
-			     vc_req->transfer_buffer_length);
-	if (!urb) {
-		ERROR("couldn't allocate urb");
-		return USBD_STATUS_NO_MEMORY;
+		if (usb_pipein(pipe) &&
+		    (!(vc_req->transfer_flags & USBD_SHORT_TRANSFER_OK))) {
+			USBTRACE("short not ok");
+			urb->transfer_flags |= URB_SHORT_NOT_OK;
+		}
+
+		dr = kmalloc(sizeof(*dr), GFP_ATOMIC);
+		if (!dr) {
+			ERROR("couldn't allocate memory");
+			wrap_free_urb(urb);
+			return USBD_STATUS_NO_MEMORY;
+		}
+		memset(dr, 0, sizeof(*dr));
+		dr->bRequestType = req_type;
+		dr->bRequest = vc_req->request;
+		dr->wValue = cpu_to_le16(vc_req->value);
+		dr->wIndex = cpu_to_le16(vc_req->index);
+		dr->wLength = cpu_to_le16(vc_req->transfer_buffer_length);
+
+		usb_fill_control_urb(urb, udev, pipe, (unsigned char *)dr,
+				     urb->transfer_buffer,
+				     vc_req->transfer_buffer_length,
+				     wrap_urb_complete, urb->context);
+		status = USBD_STATUS_PENDING;
 	}
-
-	if (usb_pipein(pipe) &&
-	    (!(vc_req->transfer_flags & USBD_SHORT_TRANSFER_OK))) {
-		USBTRACE("short not ok");
-		urb->transfer_flags |= URB_SHORT_NOT_OK;
-	}
-
-	dr = kmalloc(sizeof(*dr), GFP_ATOMIC);
-	if (!dr) {
-		ERROR("couldn't allocate memory");
-		wrap_free_urb(urb);
-		return USBD_STATUS_NO_MEMORY;
-	}
-	memset(dr, 0, sizeof(*dr));
-	dr->bRequestType = req_type;
-	dr->bRequest = vc_req->request;
-	dr->wValue = cpu_to_le16(vc_req->value);
-	dr->wIndex = cpu_to_le16(vc_req->index);
-	dr->wLength = cpu_to_le16(vc_req->transfer_buffer_length);
-
-	usb_fill_control_urb(urb, udev, pipe, (unsigned char *)dr,
-			     urb->transfer_buffer,
-			     vc_req->transfer_buffer_length,
-			     wrap_urb_complete, urb->context);
-
-	status = USBD_STATUS_PENDING;
+	USBTRACE("status: %08X", status);
 	USBEXIT(return status);
 }
 
@@ -802,10 +817,10 @@ static USBD_STATUS wrap_abort_pipe(struct usb_device *udev, struct irp *irp)
 {
 	union nt_urb *nt_urb;
 	usbd_pipe_handle pipe_handle;
-	unsigned long flags;
 	struct urb *urb;
 	struct wrap_urb *wrap_urb;
 	struct wrap_device *wd;
+	KIRQL irql;
 
 	USBENTER("irp = %p", irp);
 	wd = irp->wd;
@@ -813,7 +828,7 @@ static USBD_STATUS wrap_abort_pipe(struct usb_device *udev, struct irp *irp)
 	pipe_handle = nt_urb->pipe_req.pipe_handle;
 	nt_urb = URB_FROM_IRP(irp);
 	while (1) {
-		kspin_lock_irqsave(&irp_cancel_lock, flags);
+		irql = nt_spin_lock_irql(&irp_cancel_lock, DISPATCH_LEVEL);
 		urb = NULL;
 		nt_list_for_each_entry(wrap_urb, &wd->usb.wrap_urb_list,
 				       list) {
@@ -825,7 +840,7 @@ static USBD_STATUS wrap_abort_pipe(struct usb_device *udev, struct irp *irp)
 				break;
 			}
 		}
-		kspin_unlock_irqrestore(&irp_cancel_lock, flags);
+		nt_spin_unlock_irql(&irp_cancel_lock, irql);
 		if (urb) {
 			wrap_cancel_urb(urb);
 			USBTRACE("canceled urb: %p", urb);
@@ -871,13 +886,11 @@ static void set_intf_pipe_info(struct usb_interface *usb_intf,
 		pipe->wMaxPacketSize = ep->wMaxPacketSize;
 		pipe->bEndpointAddress = ep->bEndpointAddress;
 		pipe->bInterval = ep->bInterval;
-		pipe->type =
-			ep->bmAttributes & USB_ENDPOINT_XFERTYPE_MASK;
+		pipe->type = ep->bmAttributes & USB_ENDPOINT_XFERTYPE_MASK;
 		pipe->handle = ep;
 		USBTRACE("%d: ep 0x%x, type %d, pkt_sz %d, intv %d,"
-			 "type: %d, handle %p", i,
-			 ep->bEndpointAddress, ep->bmAttributes,
-			 ep->wMaxPacketSize, ep->bInterval,
+			 "type: %d, handle %p", i, ep->bEndpointAddress,
+			 ep->bmAttributes, ep->wMaxPacketSize, ep->bInterval,
 			 pipe->type, pipe->handle);
 	}
 }
@@ -902,7 +915,11 @@ static USBD_STATUS wrap_select_configuration(struct wrap_device *wd,
 		ret = usb_control_msg(udev, usb_sndctrlpipe(udev, 0),
 				      USB_REQ_SET_CONFIGURATION, 0,
 				      0, 0, NULL, 0, USB_CTRL_SET_TIMEOUT);
-		return wrap_urb_status(ret);
+		USBTRACE("ret: %d", ret);
+		if (ret < 0)
+			return wrap_urb_status(ret);
+		else
+			return USBD_STATUS_SUCCESS;
 	}
 
 	USBTRACE("conf: %d, type: %d, length: %d, numif: %d, attr: %08x",
@@ -914,6 +931,7 @@ static USBD_STATUS wrap_select_configuration(struct wrap_device *wd,
 			      USB_REQ_SET_CONFIGURATION, 0,
 			      config->bConfigurationValue, 0,
 			      NULL, 0, USB_CTRL_SET_TIMEOUT);
+	USBTRACE("ret: %d", ret);
 	if (ret < 0) {
 		ERROR("ret: %d", ret);
 		return wrap_urb_status(ret);
@@ -1047,8 +1065,8 @@ static USBD_STATUS wrap_process_nt_urb(struct irp *irp)
 
 	DUMP_IRP(irp);
 	switch (nt_urb->header.function) {
-		/* bulk/int and vendor/class urbs are submitted
-		 * asynchronously later by pdoDispatchDeviceControl */
+		/* bulk/int urbs are submitted asynchronously later by
+		 * pdoDispatchDeviceControl */
 	case URB_FUNCTION_BULK_OR_INTERRUPT_TRANSFER:
 		USBTRACE("submitting bulk/int irp: %p", irp);
 		status = wrap_bulk_or_intr_trans(irp);
@@ -1135,10 +1153,8 @@ static USBD_STATUS wrap_get_port_status(struct irp *irp)
 		USBTRACE("state: %d, *status: %08X", state, *status);
 	}
 #else
-	{
-		/* TODO: how to get current status? */
-		*status = USBD_PORT_CONNECTED | USBD_PORT_ENABLED;
-	}
+	/* TODO: how to get current status? */
+	*status = USBD_PORT_CONNECTED | USBD_PORT_ENABLED;
 #endif
 	return USBD_STATUS_SUCCESS;
 }
@@ -1177,7 +1193,7 @@ NTSTATUS wrap_submit_irp(struct device_object *pdo, struct irp *irp)
 		status = USBD_STATUS_NOT_SUPPORTED;
 		break;
 	default:
- 		ERROR("ioctl %08X NOT IMPLEMENTED", irp_sl->params.ioctl.code);
+		ERROR("ioctl %08X NOT IMPLEMENTED", irp_sl->params.ioctl.code);
 		status = USBD_STATUS_NOT_SUPPORTED;
 		break;
 	}
@@ -1413,14 +1429,7 @@ USBD_InterfaceQueryBusInformation(void *context, ULONG level, void *buf,
 
 	bus = wd->usb.udev->bus;
 	bus_info = buf;
-#if 0
-	/* TODO: implement this */
-	bus_info->TotalBandwidth = 0;
-	bus_info->ConsumedBandwidth =
-		bus->bandwidth_allocated / 1000000;
-#else
 	UNIMPL();
-#endif
 	USBEXIT(return STATUS_NOT_IMPLEMENTED);
 }
 
