@@ -43,6 +43,8 @@
 #define STATUS_CANCELLED                0xC0000120
 #define STATUS_DEVICE_REMOVED		0xC00002B6
 
+#define STATUS_BUFFER_OVERFLOW		0x80000005
+
 #define SL_PENDING_RETURNED		0x01
 #define CALL_ON_CANCEL			0x20
 #define CALL_ON_SUCCESS			0x40
@@ -116,7 +118,7 @@
 
 #define KI_USER_SHARED_DATA 0xfffff78000000000
 
-#else 
+#else
 
 #define STDCALL __attribute__((__stdcall__, regparm(0)))
 #define _FASTCALL __attribute__((__stdcall__)) __attribute__((regparm (3)))
@@ -172,6 +174,8 @@ typedef ULONG_PTR SIZE_T;
 typedef ULONG_PTR KAFFINITY;
 typedef ULONG ACCESS_MASK;
 
+typedef unsigned long PFN_NUMBER;
+
 /* non-negative numbers indicate success */
 #define NT_SUCCESS(status)  ((NTSTATUS)(status) >= 0)
 
@@ -205,7 +209,7 @@ struct nt_list {
 	struct nt_list *prev;
 };
 
-typedef ULONG_PTR KSPIN_LOCK;
+typedef ULONG_PTR NT_SPIN_LOCK;
 
 struct kdpc;
 typedef STDCALL void (*DPC)(struct kdpc *kdpc, void *ctx, void *arg1,
@@ -222,7 +226,7 @@ struct kdpc {
 	void *ctx;
 	void *arg1;
 	void *arg2;
-	KSPIN_LOCK *lock;
+	NT_SPIN_LOCK *lock;
 };
 
 enum pool_type {
@@ -281,16 +285,19 @@ struct mdl {
 
 #define MmGetMdlBaseVa(mdl) ((mdl)->startva)
 #define MmGetMdlByteCount(mdl) ((mdl)->bytecount)
-#define MmGetMdlVirtualAddress(mdl) ((void *)((char *)(mdl)->startva +	\
-					      (mdl)->byteoffset))
+#define MmGetMdlVirtualAddress(mdl) ((mdl)->startva +	\
+				     (mdl)->byteoffset)
 #define MmGetMdlByteOffset(mdl) ((mdl)->byteoffset)
 #define MmGetSystemAddressForMdl(mdl) ((mdl)->mappedsystemva)
+#define MmGetSystemAddressForMdlSafe(mdl, priority) ((mdl)->mappedsystemva)
+#define MmGetMdlPfnArray(mdl) ((PFN_NUMBER *)(mdl + 1))
 #define MmInitializeMdl(mdl, baseva, length) {				\
 		(mdl)->next = NULL;					\
 		(mdl)->size = MmSizeOfMdl(baseva, length);		\
 		(mdl)->flags = 0;					\
-		(mdl)->startva = baseva;				\
-		(mdl)->byteoffset = 0;					\
+ 		(mdl)->startva =					\
+			(void *)((unsigned long)baseva & PAGE_MASK);	\
+ 		(mdl)->byteoffset = (ULONG)offset_in_page(baseva);	\
 		(mdl)->bytecount = length;				\
 	}
 
@@ -304,7 +311,7 @@ struct kdevice_queue {
 	USHORT type;
 	USHORT size;
 	struct nt_list list;
-	KSPIN_LOCK lock;
+	NT_SPIN_LOCK lock;
 	BOOLEAN busy;
 };
 
@@ -386,7 +393,7 @@ struct nt_thread {
 	int pid;
 	struct task_struct *task;
 	struct nt_list irps;
-	KSPIN_LOCK lock;
+	NT_SPIN_LOCK lock;
 	wait_queue_head_t event_wq;
 	int event_wait_done;
 };
@@ -1109,7 +1116,7 @@ struct npaged_lookaside_list {
 	} u3;
 	ULONG pad[2];
 #ifndef X86_64
-	KSPIN_LOCK obsolete;
+	NT_SPIN_LOCK obsolete;
 #endif
 };
 
@@ -1141,8 +1148,8 @@ typedef BOOLEAN (*PKSYNCHRONIZE_ROUTINE)(void *context) STDCALL;
 struct kinterrupt {
 	ULONG vector;
 	KAFFINITY processor_enable_mask;
-	KSPIN_LOCK lock;
-	KSPIN_LOCK *actual_lock;
+	NT_SPIN_LOCK lock;
+	NT_SPIN_LOCK *actual_lock;
 	BOOLEAN shareable;
 	BOOLEAN floating_save;
 	CHAR processor_number;
@@ -1185,7 +1192,7 @@ struct callback_func {
 };
 
 struct callback_object {
-	KSPIN_LOCK lock;
+	NT_SPIN_LOCK lock;
 	struct nt_list list;
 	struct nt_list callback_funcs;
 	BOOLEAN allow_multiple_callbacks;
