@@ -19,12 +19,13 @@
 #include "loader.h"
 #include "wrapndis.h"
 
-extern KSPIN_LOCK loader_lock;
+extern NT_SPIN_LOCK loader_lock;
 extern struct wrap_device *wrap_devices;
 extern struct nt_list wrap_drivers;
 
 extern char *if_name;
 extern int hangcheck_interval;
+extern int disable_stats;
 extern const struct iw_handler_def ndis_handler_def;
 
 static int set_packet_filter(struct wrap_ndis_device *wnd,
@@ -59,12 +60,12 @@ static inline int ndis_wait_pending_completion(struct wrap_ndis_device *wnd)
 /* MiniportReset */
 NDIS_STATUS miniport_reset(struct wrap_ndis_device *wnd)
 {
-	KIRQL irql;
 	NDIS_STATUS res = 0;
 	struct miniport_char *miniport;
 	UINT cur_lookahead;
 	UINT max_lookahead;
 	BOOLEAN reset_address;
+	KIRQL irql;
 
 	TRACEENTER2("wnd: %p", wnd);
 
@@ -96,14 +97,13 @@ NDIS_STATUS miniport_reset(struct wrap_ndis_device *wnd)
 	}
 	DBGTRACE2("reset: res = %08X, reset status = %08X",
 		  res, reset_address);
-
+	up(&wnd->ndis_comm_mutex);
 	if (res == NDIS_STATUS_SUCCESS && reset_address) {
 		wnd->nmb->cur_lookahead = cur_lookahead;
 		wnd->nmb->max_lookahead = max_lookahead;
 		set_packet_filter(wnd, wnd->packet_filter);
 //		set_multicast_list(wnd->net_dev);
 	}
-	up(&wnd->ndis_comm_mutex);
 
 	TRACEEXIT3(return res);
 }
@@ -274,7 +274,7 @@ static void miniport_halt(struct wrap_ndis_device *wnd)
 	miniport = &wnd->wd->driver->ndis_driver->miniport;
 	DBGTRACE1("driver halt is at %p", miniport->miniport_halt);
 
-	DBGTRACE2("task: %p, pid: %d", get_current(), get_current()->pid);
+	DBGTRACE2("task: %p, pid: %d", current, current->pid);
 	LIN2WIN1(miniport->miniport_halt, wnd->nmb->adapter_ctx);
 	clear_bit(HW_INITIALIZED, &wnd->hw_status);
 
@@ -307,7 +307,7 @@ static NDIS_STATUS miniport_pnp_event(struct wrap_ndis_device *wnd,
 	case NdisDevicePnPEventPowerProfileChanged:
 		if (!miniport->pnp_event_notify) {
 			DBGTRACE1("Windows driver %s doesn't support "
-				"MiniportPnpEventNotify",
+				  "MiniportPnpEventNotify",
 				  wnd->wd->driver->name);
 			return NDIS_STATUS_FAILURE;
 		}
@@ -469,11 +469,6 @@ static void free_send_packet(struct wrap_ndis_device *wnd,
 	struct ndis_packet_oob_data *oob_data;
 
 	TRACEENTER3("packet: %p", packet);
-	if (!packet) {
-		ERROR("illegal packet from %p", wnd);
-		return;
-	}
-
 	buffer = packet->private.buffer_head;
 	oob_data = NDIS_PACKET_OOB_DATA(packet);
 	if (wnd->use_sg_dma)
@@ -484,7 +479,7 @@ static void free_send_packet(struct wrap_ndis_device *wnd,
 
 	DBGTRACE3("freeing buffer %p", buffer);
 	NdisFreeBuffer(buffer);
-	dev_kfree_skb(oob_data->skb);
+	dev_kfree_skb_any(oob_data->skb);
 
 	DBGTRACE3("freeing packet %p", packet);
 	NdisFreePacket(packet);
@@ -577,10 +572,12 @@ static void xmit_worker(void *param)
 
 	/* some drivers e.g., new RT2500 driver, crash if any packets
 	 * are sent when the card is not associated */
-	irql = kspin_lock_irql(&wnd->xmit_lock, DISPATCH_LEVEL);
 	while (wnd->send_ok) {
-		if (wnd->xmit_ring_pending == 0)
+		irql = nt_spin_lock_irql(&wnd->xmit_lock, DISPATCH_LEVEL);
+		if (wnd->xmit_ring_pending == 0) {
+			nt_spin_unlock_irql(&wnd->xmit_lock, irql);
 			break;
+		}
 		n = send_packets(wnd, wnd->xmit_ring_start,
 				 wnd->xmit_ring_pending);
 		wnd->xmit_ring_start =
@@ -588,8 +585,8 @@ static void xmit_worker(void *param)
 		wnd->xmit_ring_pending -= n;
 		if (netif_queue_stopped(wnd->net_dev) && n > 0)
 			netif_wake_queue(wnd->net_dev);
+		nt_spin_unlock_irql(&wnd->xmit_lock, irql);
 	}
-	kspin_unlock_irql(&wnd->xmit_lock, irql);
 
 	TRACEEXIT3(return);
 }
@@ -597,16 +594,16 @@ static void xmit_worker(void *param)
 /*
  * Free and unmap packet created in xmit
  */
-void sendpacket_done(struct wrap_ndis_device *wnd, struct ndis_packet *packet)
+void sendpacket_done(struct wrap_ndis_device *wnd,
+		     struct ndis_packet *packet)
 {
-	KIRQL irql;
 
 	TRACEENTER3("%p", packet);
-	irql = kspin_lock_irql(&wnd->send_packet_done_lock, DISPATCH_LEVEL);
+	nt_spin_lock(&wnd->send_packet_done_lock);
 	wnd->stats.tx_bytes += packet->private.len;
 	wnd->stats.tx_packets++;
+	nt_spin_unlock(&wnd->send_packet_done_lock);
 	free_send_packet(wnd, packet);
-	kspin_unlock_irql(&wnd->send_packet_done_lock, irql);
 	TRACEEXIT3(return);
 }
 
@@ -636,14 +633,14 @@ static int start_xmit(struct sk_buff *skb, struct net_device *dev)
 	}
 	oob_data = NDIS_PACKET_OOB_DATA(packet);
 	oob_data->skb = skb;
-	kspin_lock(&wnd->xmit_lock);
+	nt_spin_lock(&wnd->xmit_lock);
 	xmit_ring_next_slot = (wnd->xmit_ring_start +
 			       wnd->xmit_ring_pending) % XMIT_RING_SIZE;
 	wnd->xmit_ring[xmit_ring_next_slot] = packet;
 	wnd->xmit_ring_pending++;
 	if (wnd->xmit_ring_pending == XMIT_RING_SIZE)
 		netif_stop_queue(wnd->net_dev);
-	kspin_unlock(&wnd->xmit_lock);
+	nt_spin_unlock(&wnd->xmit_lock);
 
 	schedule_work(&wnd->xmit_work);
 
@@ -692,7 +689,7 @@ static int ndis_open(struct net_device *dev)
 	if (dev->flags & IFF_PROMISC)
 		packet_filter |= NDIS_PACKET_TYPE_PROMISCUOUS |
 			NDIS_PACKET_TYPE_ALL_LOCAL;
-	if (set_packet_filter(wnd, packet_filter) && 
+	if (set_packet_filter(wnd, packet_filter) &&
 	    packet_filter & NDIS_PACKET_TYPE_PROMISCUOUS) {
 		DBGTRACE1("couldn't add packet filter %x", packet_filter);
 		packet_filter &= ~NDIS_PACKET_TYPE_PROMISCUOUS;
@@ -949,9 +946,6 @@ static void hangcheck_proc(unsigned long data)
 	struct wrap_ndis_device *wnd = (struct wrap_ndis_device *)data;
 
 	TRACEENTER3("");
-	if (wnd->hangcheck_interval <= 0)
-		return;
-
 	set_bit(HANGCHECK, &wnd->wrap_ndis_work);
 	schedule_work(&wnd->wrap_ndis_worker);
 
@@ -964,8 +958,10 @@ static void hangcheck_proc(unsigned long data)
 void hangcheck_add(struct wrap_ndis_device *wnd)
 {
 	if (!wnd->wd->driver->ndis_driver->miniport.hangcheck ||
-	    wnd->hangcheck_interval <= 0)
+	    hangcheck_interval < 0)
 		return;
+	if (hangcheck_interval > 0)
+		wnd->hangcheck_interval = hangcheck_interval * HZ;
 	init_timer(&wnd->hangcheck_timer);
 	wnd->hangcheck_timer.data = (unsigned long)wnd;
 	wnd->hangcheck_timer.function = &hangcheck_proc;
@@ -1531,7 +1527,7 @@ static NDIS_STATUS ndis_remove_device(struct wrap_ndis_device *wnd)
 	netif_carrier_off(wnd->net_dev);
 	wrap_procfs_remove_ndis_device(wnd);
 	/* throw away pending packets */
-	irql = kspin_lock_irql(&wnd->xmit_lock, DISPATCH_LEVEL);
+	irql = nt_spin_lock_irql(&wnd->xmit_lock, DISPATCH_LEVEL);
 	while (wnd->xmit_ring_pending) {
 		struct ndis_packet *packet;
 
@@ -1541,7 +1537,7 @@ static NDIS_STATUS ndis_remove_device(struct wrap_ndis_device *wnd)
 			(wnd->xmit_ring_start + 1) % XMIT_RING_SIZE;
 		wnd->xmit_ring_pending--;
 	}
-	kspin_unlock_irql(&wnd->xmit_lock, irql);
+	nt_spin_unlock_irql(&wnd->xmit_lock, irql);
 	miniport_halt(wnd);
 	ndis_exit_device(wnd);
 	/* flush_scheduled_work here causes crash with 2.4 kernels */
@@ -1620,7 +1616,7 @@ static STDCALL NTSTATUS NdisAddDevice(struct driver_object *drv_obj,
 	init_nmb_functions(nmb);
 	wnd->net_dev = net_dev;
 	wnd->ndis_irq = NULL;
-	kspin_lock_init(&wnd->xmit_lock);
+	nt_spin_lock_init(&wnd->xmit_lock);
 	init_MUTEX(&wnd->ndis_comm_mutex);
 	init_waitqueue_head(&wnd->ndis_comm_wq);
 	wnd->ndis_comm_done = 0;
@@ -1629,16 +1625,15 @@ static STDCALL NTSTATUS NdisAddDevice(struct driver_object *drv_obj,
 	INIT_WORK(&wnd->xmit_work, xmit_worker, wnd);
 	wnd->xmit_ring_start = 0;
 	wnd->xmit_ring_pending = 0;
-	kspin_lock_init(&wnd->send_packet_done_lock);
+	nt_spin_lock_init(&wnd->send_packet_done_lock);
 	wnd->encr_mode = Ndis802_11EncryptionDisabled;
 	wnd->auth_mode = Ndis802_11AuthModeOpen;
 	wnd->capa.encr = 0;
 	wnd->capa.auth = 0;
 	wnd->attributes = 0;
-	wnd->map_count = 0;
-	wnd->map_dma_addr = NULL;
+	wnd->dma_map_count = 0;
+	wnd->dma_map_addr = NULL;
 	wnd->nick[0] = 0;
-	wnd->hangcheck_interval = hangcheck_interval;
 	init_timer(&wnd->hangcheck_timer);
 	wnd->scan_timestamp = 0;
 	init_timer(&wnd->stats_timer);
